#ifndef UTIL_H
#define UTIL_H

#include <string>
#include <ctime>
#include <vector>

#include <CL/opencl.h>

void error(std::string str, int code=-1);

void randomizeVector(std::vector<float>& vec, float min, float max);

/// If d>0.5 then returns 1, else returns 0.
int classify(float d);


class Timer
{
public:
    Timer();
    /** Starts the timer. */
    void start();
    /** Returns time in seconds. */
    double stop();
    /** Returns time in seconds.*/
    double elapsed();

    /** Just returns from functions without doing anything. */
    void disable();

    /** Print out a time in seconds. Text specifies what is timed. */
    void print(std::string text);

private:
    unsigned long long started_;
    unsigned long long stopped_;
    bool running_;
    bool disabled_;
};

class Logger
{
public:
    static Logger* getInstance();
    ~Logger();
protected:
    Logger(); // constructor
private:
    static Logger* logger_;
};


/** Returns true if line contains paramName, and value is stored in result.
    For reading in configuration files. */
bool ParseLine(std::string line, std::string paramName, double &result);

void openclPrintDeviceInfo(cl_device_id device);

void openclPrintAllDevicesInfo();

std::string readFile(std::string fileName);

cl_context openclCreateContextFromType(cl_device_type device_type, cl_device_id *device);

cl_program openclCreateProgramFromSource(cl_context& context, std::string source);
cl_program openclCreateProgramFromFile(cl_context& context, std::string fileName);

void openclBuildProgram(cl_program& program, cl_device_id& device);

std::string findAndReplace(std::string input, std::string find, std::string replace);

#endif // UTIL_H
